EnableExplicit



;IncludeFile "Res\Vector.pb"
Declare Vector_FreeGarbageCollector()




;{- Started and Ended TailBite Functions
ProcedureDLL Vector_Init()
  Global NewList Vector__GarbageCollector()
EndProcedure
ProcedureDLL Vector_End()
  Vector_FreeGarbageCollector()
EndProcedure
;}




;Procedure ValidatePBHeap(LineNumber)
;  Protected Heap
;  !mov eax, dword [_PB_MemoryBase]
;  !mov [p.v_Heap], eax
;  If HeapValidate_(Heap, 0, 0) = 0
;    MessageRequester("Error", "PB Memory Heap invalid at Line: "+Str(LineNumber))
;  EndIf 
;EndProcedure 

;Macro _validate
;  ValidatePBHeap(#PB_Compiler_Line)
;EndMacro 


;/------------------------------------------\
;|                                          |
;-              Privates Functions        
;|                                          |
;\------------------------------------------/


;{- Vector__RaiseError
;   Local Function for displaying errors
;   When this function is called, the program will be exiting
Procedure Vector__RaiseError(Message$, Function$ = #NULL$, Flag = #MB_ICONERROR)
  Protected title$
  If Function$
    title$ = #Vector_ErrorTitle + " - " + Function$ + "()"
  Else
    title$ = #Vector_ErrorTitle
  EndIf
  MessageRequester(title$, Message$, Flag)
  End
EndProcedure
;}


Macro Vector_TestInit(vector_)
  vector_\Initilized = #Vector_Flag_Allocated
EndMacro

Macro Vector_TestNotInit(vector_)
  vector_\Initilized <> #Vector_Flag_Allocated
EndMacro

Macro Vector_SameType(v1_, v2_)
  v1_\Constructor = v2_\Constructor And v1_\Destructor = v2_\Destructor And v1_\Type = v2_\Type And v1_\Comparator = v2_\Comparator And v1_\Size = v2_\Size
EndMacro

;{- Vector__IsInitialized (Macro)
;   Test if the Vector has been created
;   Otherwise the program will be quiting with an error
Macro Vector__IsInitialized(vector, FunctionName)
  If vector = #Null
    Vector__RaiseError("Try to use Vector Functions from Null Pointer !", FunctionName)
  ElseIf Vector_TestNotInit(vector)
    Vector__RaiseError("Vector is not initialized !", FunctionName)
  EndIf
EndMacro
;}



ProcedureDLL Vector_IsInitialized(*vector.Vector)
  If *vector And Vector_TestInit(*vector)
    ProcedureReturn #True
  EndIf
EndProcedure


;{- Vector__IsType (Macro)
;   Test if the Vector has been created and if his Type is correct
;   Otherwise the program will be quiting with an error
Macro Vector__IsType(vector, type_, FunctionName)
  Vector__IsInitialized(vector, FunctionName)
  If vector\Type <> type_
    Vector__RaiseError("Not good type !", FunctionName)
  EndIf
EndMacro
;}



;{- Vector__Allocate : Vector AllocateMemory
;   If Can't return a Null Pointer, because the 'Exception' is catched
ProcedureDLL Vector__Allocate(Size.l)
  Protected *mem
  *mem = AllocateMemory(Size)
  If *mem = #Null
    Vector__RaiseError("Can't allocate memory")
  EndIf
  ProcedureReturn *mem
EndProcedure
;}



;{- Vector__GetNextVNode + Vector__GetPreviousVNode
;   Just the Next or Previous Element, or Null if *node=#Null
Procedure Vector__GetNextVNode(*node.VNode)
  If *node
    ProcedureReturn *node\NextElement
  EndIf
EndProcedure
Procedure Vector__GetPreviousVNode(*node.VNode)
  If *node
    ProcedureReturn *node\PreviousElement
  EndIf
EndProcedure
;}


;{- V__NextElement + V__PreviousElement
;   Change the CurrentElement by the Next or Previous
;   Becarefull : Their must be a CurrentElement
Macro V__NextElement(vector_)
  vector_\CurrentElement = vector_\CurrentElement\NextElement
  vector_\Index + 1
EndMacro
Macro V__PreviousElement(vector_)
  vector_\CurrentElement = vector_\CurrentElement\PreviousElement
  vector_\Index - 1
EndMacro
;}


Macro Vector__SaveContext(vector_, memory_)
  memory_ = Vector__Allocate(SizeOf(Vector))
  CopyMemory(vector_, memory_, SizeOf(Vector))
EndMacro

Macro Vector__RestoreContext(vector_, memory_)
  CopyMemory(memory_, vector_, SizeOf(Vector))
  FreeMemory(memory_)
EndMacro



;/------------------------------------------\
;|                                          |
;-   Constructors and Destructor Functions 
;|                                          |
;\------------------------------------------/


Procedure Vector_WriteString(*address.Long, Value.s = "")
  Protected BufLen.l
  
  If *address
    BufLen = Len(value) + 1
    CompilerIf #PB_Compiler_Unicode
      BufLen * 2
    CompilerEndIf
    *address\l = Vector__Allocate(BufLen)
    CopyMemory(@value, *address\l, BufLen)
  Else
    Vector__RaiseError("Pointer is Null", "Vector_WriteString")
  EndIf
EndProcedure


ProcedureDLL V__Constructor_String(*param = #Null)
  Protected *str.String, *mem.String

  If *param
    *str = @*param
    *mem = Vector__Allocate(SizeOf(String))
    *mem\s = *str\s
  Else
    *mem = Vector__Allocate(1)
    *mem\s = ""
  EndIf
  ProcedureReturn *mem
EndProcedure

;{- Default Constructor and Destructor
ProcedureDLL V__DefaultConstructor(Size.l, *param = #Null)
  Protected *mem
  *mem = Vector__Allocate(Size)
  If *param
    CopyMemory(*param, *mem, Size)
  EndIf
  ProcedureReturn *mem
EndProcedure
ProcedureDLL V__DefaultDestructor(*param)
EndProcedure
;}



Procedure V__Constructor_FileInfos(*param.Vector_FileInfos = #Null)
  Protected *mem.Vector_FileInfos
  
  *mem = Vector__Allocate(SizeOf(Vector_FileInfos))
  If *param
    ;Vector_WriteString(*mem + OffsetOf(Vector_FileInfos\ShortFileName), *param\ShortFileName)
    *mem\ShortFileName    = *param\ShortFileName      ;- Does it need to be improved to be more safe ???
    *mem\CompleteFileName = *param\CompleteFileName
    *mem\DateCreated      = *param\DateCreated
    *mem\DateAccessed     = *param\DateAccessed
    *mem\DateModified     = *param\DateModified
    *mem\FileSize         = *param\FileSize
    *mem\CustomData       = *param\CustomData
  Else
    ;*mem\ShortFileName    = ""
    ;*mem\CompleteFileName = *param\CompleteFileName
    ;*mem\DateCreated      = 0
    ;*mem\DateAccessed     = 0
    ;*mem\DateModified     = 0
    ;*mem\FileSize         = 0
    ;*mem\CustomData       = 0
  EndIf
  ProcedureReturn *mem
EndProcedure
Procedure V_Destructor_FileInfos(*param.Vector_FileInfos)
  ;If *param
  ;  FreeMemory(@*param\FileName)
  ;EndIf
EndProcedure



;/------------------------------------------\
;|                                          |
;-          Comparators Functions 
;|                                          |
;\------------------------------------------/


;{- V__Comparator_xxx : String, Byte, Long, Double, etc...
;   The differents comparators
;   For Complex Structures, users have to define their own coparators
ProcedureDLL V__Comparator_Byte(*obj1.Byte, *obj2.Byte)
  If *obj1\b = *obj2\b
    ProcedureReturn #Vector_Compare_Equal
  ElseIf *obj1\b < *obj2\b
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
ProcedureDLL V__Comparator_Word(*obj1.Word, *obj2.Word)
  If *obj1\w = *obj2\w
    ProcedureReturn #Vector_Compare_Equal
  ElseIf *obj1\w < *obj2\w
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
ProcedureDLL V__Comparator_Long(*obj1.Long, *obj2.Long)
  If *obj1\l = *obj2\l
    ProcedureReturn #Vector_Compare_Equal
  ElseIf *obj1\l < *obj2\l
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
ProcedureDLL V__Comparator_Float(*obj1.Float, *obj2.Float)
  If *obj1\f = *obj2\f
    ProcedureReturn #Vector_Compare_Equal
  ElseIf *obj1\f < *obj2\f
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
ProcedureDLL V__Comparator_Quad(*obj1.Quad, *obj2.Quad)
  If *obj1\q = *obj2\q
    ProcedureReturn #Vector_Compare_Equal
  ElseIf *obj1\q < *obj2\q
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
ProcedureDLL V__Comparator_StringCaseSensitive(*obj1.String, *obj2.String)
  ProcedureReturn CompareMemoryString(@*obj1\s, @*obj2\s, #True)
EndProcedure
ProcedureDLL V__Comparator_String(*obj1.String, *obj2.String)
  ProcedureReturn CompareMemoryString(@*obj1\s, @*obj2\s, #False)
EndProcedure
ProcedureDLL V__Comparator_Double(*obj1.Double, *obj2.Double)
  If *obj1\d = *obj2\d
    ProcedureReturn #Vector_Compare_Equal
  ElseIf *obj1\d < *obj2\d
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
;}


Macro V__Comp_FileInfo_Name()
  CompareMemoryString(@*obj1\ShortFileName, @*obj2\ShortFileName, #False)
EndMacro
Procedure V__Comparator_FileInfo_Name_U(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  ProcedureReturn V__Comp_FileInfo_Name()
EndProcedure
Procedure V__Comparator_FileInfo_Name_D(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  ProcedureReturn V__Comp_FileInfo_Name() * -1
EndProcedure
Procedure V__Comparator_FileInfo_Size_U(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\FileSize = *obj2\FileSize
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\FileSize < *obj2\FileSize
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
Procedure V__Comparator_FileInfo_Size_D(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\FileSize = *obj2\FileSize
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\FileSize > *obj2\FileSize
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
Procedure V__Comparator_FileInfo_DateC_U(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\DateCreated = *obj2\DateCreated
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\DateCreated < *obj2\DateCreated
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
Procedure V__Comparator_FileInfo_DateC_D(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\DateCreated = *obj2\DateCreated
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\DateCreated > *obj2\DateCreated
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
Procedure V__Comparator_FileInfo_DateA_U(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\DateAccessed = *obj2\DateAccessed
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\DateAccessed < *obj2\DateAccessed
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
Procedure V__Comparator_FileInfo_DateA_D(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\DateAccessed = *obj2\DateAccessed
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\DateAccessed > *obj2\DateAccessed
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
Procedure V__Comparator_FileInfo_DateM_U(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\DateModified = *obj2\DateModified
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\DateModified < *obj2\DateModified
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure
Procedure V__Comparator_FileInfo_DateM_D(*obj1.Vector_FileInfos, *obj2.Vector_FileInfos)
  If *obj1\DateModified = *obj2\DateModified
    ProcedureReturn V__Comp_FileInfo_Name()
  ElseIf *obj1\DateModified > *obj2\DateModified
    ProcedureReturn #Vector_Compare_Less
  Else
    ProcedureReturn #Vector_Compare_More
  EndIf
EndProcedure





;/------------------------------------------\
;|                                          |
;-              Mutex Functions
;|                                          |
;\------------------------------------------/


Macro Vector__Lock(MemAddress)
  ;LockMutex(MemAddress)
EndMacro
Macro Vector__UnLock(MemAddress)
  ;UnlockMutex(MemAddress)
EndMacro

ProcedureDLL Vector_Lock(*vector.Vector)
  ;MessageRequester("Vector__Lock()", Str(*vector))
  Vector__Lock(*vector)
EndProcedure
ProcedureDLL Vector_UnLock(*vector.Vector)
  ;MessageRequester("Vector__UnLock()", Str(*vector))
  Vector__UnLock(*vector)
EndProcedure




;/------------------------------------------\
;|                                          |
;-             Get/Set Functions 
;|                                          |
;\------------------------------------------/

Declare V__SelectElement(*vector.Vector, index.l, Fail.l = #Null, FonctionName.s = #NULL$)

Macro V__SetPosition(vector_, current_, index_)
  vector_\CurrentElement  = current_
  vector_\Index           = index_
EndMacro

ProcedureDLL Vector_Count(*vector.Vector)             ; Count Nodes
  Vector__IsInitialized(*vector, "Vector_Count")
  ProcedureReturn *vector\Count
EndProcedure
ProcedureDLL Vector_Index(*vector.Vector)             ; Position
  Vector__IsInitialized(*vector, "Vector_Index")
  ProcedureReturn *vector\Index
EndProcedure
ProcedureDLL Vector_GetCurrentElement(*vector.Vector) ; Current Element
  Vector__IsInitialized(*vector, "Vector_CurrentElement")
  If *vector\CurrentElement
    ProcedureReturn *vector\CurrentElement\Value
  Else
    Vector__RaiseError("There are no current element !", "Vector_GetCurrentElement")
  EndIf
EndProcedure
ProcedureDLL Vector_GetFirstElement(*vector.Vector)   ; Firts Element
  Vector__IsInitialized(*vector, "Vector_GetFirstElement")
  Vector__Lock(*vector)
  V__SelectElement(*vector, 0, #True, "Vector_GetFirstElement")
  Vector__UnLock(*vector)
  ProcedureReturn *vector\CurrentElement\Value
EndProcedure
ProcedureDLL Vector_GetLastElement(*vector.Vector)    ; Last Element
  Vector__IsInitialized(*vector, "Vector_GetLastElement")
  Vector__Lock(*vector)
  V__SelectElement(*vector, *vector\Count - 1, #True, "Vector_GetLastElement")
  Vector__UnLock(*vector)
  ProcedureReturn *vector\CurrentElement\Value
EndProcedure
ProcedureDLL Vector_SetCaseSensitiveState(*vector.Vector, state.l)
  Vector__IsInitialized(*vector, "Vector_SetCaseSensitiveState")
  If *vector\Type = #String
    If state
      *vector\Constructor = @V__Comparator_StringCaseSensitive()
    Else
      *vector\Constructor = @V__Constructor_String()
    EndIf
  Else
    Vector__RaiseError("This is not a String Vector", "Vector_SetCaseSensitiveState")
  EndIf
EndProcedure





;/------------------------------------------\
;|                                          |
;-               Moves Functions 
;|                                          |
;\------------------------------------------/

Macro Vector__Reset(vector_)
  vector_\CurrentElement = #Null
  vector_\Index = -1
EndMacro
ProcedureDLL Vector_Reset(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_Reset")
  With *vector
    Vector__Lock(\Mutex)
    Vector__Reset(*vector)
    Vector__UnLock(\Mutex)
    ProcedureReturn \CurrentElement
  EndWith
EndProcedure

ProcedureDLL Vector_FirstElement(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_FirstElement")
  With *vector
    Vector__Lock(\Mutex)
    \CurrentElement = \FirstElement
    \Index = 0
    Vector__UnLock(\Mutex)
    ProcedureReturn \CurrentElement
  EndWith
EndProcedure

ProcedureDLL Vector_LastElement(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_LastElement")
  With *vector
    Vector__Lock(\Mutex)
    \CurrentElement = \LastElement
    \Index = \Count - 1
    Vector__UnLock(\Mutex)
    ProcedureReturn \CurrentElement
  EndWith
EndProcedure


Procedure Vector__NextElement(*vector.Vector)
  With *vector
    If \Count And \Index = -1
      \CurrentElement = \FirstElement
      \Index = 0
    ElseIf \CurrentElement
      V__NextElement(*vector)
    EndIf
    ProcedureReturn \CurrentElement
  EndWith
EndProcedure
ProcedureDLL Vector_NextElement(*vector.Vector)
  Protected retour

  Vector__IsInitialized(*vector, "Vector_NextElement")
  With *vector
    Vector__Lock(\Mutex)
    retour = Vector__NextElement(*vector)
    Vector__UnLock(\Mutex)
    ProcedureReturn retour
  EndWith
EndProcedure

Procedure Vector__PreviousElement(*vector.Vector)
  With *vector
    If \Count = \Index
      \CurrentElement = \LastElement
      \Index - 1
    ElseIf \CurrentElement
      V__PreviousElement(*vector)
    EndIf
    ProcedureReturn \CurrentElement
  EndWith
EndProcedure
ProcedureDLL Vector_PreviousElement(*vector.Vector)
  Protected retour

  Vector__IsInitialized(*vector, "Vector_PreviousElement")
  With *vector
    Vector__Lock(\Mutex)
    retour = Vector__PreviousElement(*vector)
    Vector__UnLock(\Mutex)
    ProcedureReturn retour
  EndWith
EndProcedure

Procedure V__SelectElement(*vector.Vector, index.l, Fail.l = #Null, FonctionName.s = #NULL$)
  Protected dist.l, index_.l, *node_.VNode
  
  If FonctionName = #NULL$
    FonctionName = "Vector_SelectElement"
  EndIf
  With *vector
    If Index < -1 Or Index >= \Count
      If Fail
        Vector__RaiseError("Index out of bounds for Vector_Select()", FonctionName)
      Else
        Debug FonctionName+"() : Index out of bound ("+Str(Index)+"/"+Str(*vector\Count)+")"
        \CurrentElement = #Null
        \Index = 0
        ProcedureReturn #False
      EndIf
    EndIf
    dist = Abs(\Index - Index)
    If dist > Index
      ;Debug "Search from the begin !"
      dist = Index
      \Index = 0
      \CurrentElement = \FirstElement
    EndIf
    If dist > \Count - Index
      ;Debug "Search from the end !"
      \Index = \Count - 1
      \CurrentElement = \LastElement
    EndIf
    If Index = -1 Or Index = \Count
      \CurrentElement = #Null
      \Index = Index
    EndIf
    If \Index <> Index
      If \CurrentElement = #Null
        If \Index = -1
          \Index = 0
          \CurrentElement = \FirstElement
        Else
          \Index = \Count - 1
          \CurrentElement = \LastElement
        EndIf
      EndIf
      Repeat
        If \Index < Index
          V__NextElement(*vector)
        Else
          V__PreviousElement(*vector)
        EndIf
      Until \Index = Index
    EndIf
    ProcedureReturn \CurrentElement
  EndWith
EndProcedure


ProcedureDLL Vector_SelectElement(*vector.Vector, index.l)
  Protected retour

  Vector__IsInitialized(*vector, "Vector_SelectElement")
  Vector__Lock(*vector\Mutex)
  retour = V__SelectElement(*vector, Index, #Null)
  Vector__UnLock(*vector\Mutex)
  ProcedureReturn retour
EndProcedure
ProcedureDLL Vector_SelectElement2(*vector.Vector, index.l, Fail.l)
  Protected retour

  Vector__IsInitialized(*vector, "Vector_SelectElement")
  Vector__Lock(*vector\Mutex)
  retour = V__SelectElement(*vector, Index, Fail)
  Vector__UnLock(*vector\Mutex)
  ProcedureReturn retour
EndProcedure


;/------------------------------------------\
;|                                          |
;-              Get/Set Functions 
;|                                          |
;\------------------------------------------/


Macro V__Get(vector_)
  vector_\CurrentElement\Value
EndMacro


ProcedureDLL Vector_Get(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_GetElement")
  Vector__Lock(*vector\Mutex)
  If *vector\CurrentElement
    Vector__UnLock(*vector\Mutex)
    ProcedureReturn V__Get(*vector)
  EndIf
  Vector__UnLock(*vector\Mutex)
EndProcedure
ProcedureDLL Vector_Get2(*vector.Vector, index.l)
  Vector__IsInitialized(*vector, "Vector_GetElement")
  Vector__Lock(*vector\Mutex)
  If Index <> #Vector_CurrentElement
    If V__SelectElement(*vector, Index, #Null, "Vector_GetElement") = #Null
      ProcedureReturn #Null
    EndIf
  EndIf
  If *vector\CurrentElement
    Vector__UnLock(*vector\Mutex)
    ProcedureReturn V__Get(*vector)
  EndIf
  Vector__UnLock(*vector\Mutex)
EndProcedure
ProcedureDLL Vector_Get3(*vector.Vector, index.l, Fail.l)
  Vector__IsInitialized(*vector, "Vector_GetElement")
  Vector__Lock(*vector\Mutex)
  If Index <> #Vector_CurrentElement
    If V__SelectElement(*vector, Index, Fail, "Vector_GetElement") = #Null
      ProcedureReturn #Null
    EndIf
  EndIf
  If *vector\CurrentElement
    Vector__UnLock(*vector\Mutex)
    ProcedureReturn V__Get(*vector)
  Else
    Vector__RaiseError("The current element is a Null Pointer !", "Vector_GetElement")
  EndIf
  Vector__UnLock(*vector\Mutex)
EndProcedure

ProcedureDLL Vector_Set(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_SetElement")
  Vector__Lock(*vector\Mutex)
  With *vector
    If *vector\CurrentElement
      CallFunctionFast(\Destructor, \CurrentElement\Value)
      \CurrentElement\Value = CallFunctionFast(\Constructor, #Null)
    Else
      Vector__RaiseError("Null Pointer !", "Vector_SetElement")
    EndIf
  EndWith
  Vector__UnLock(*vector\Mutex)
EndProcedure
ProcedureDLL Vector_Set2(*vector.Vector, *Object)
  Vector__IsInitialized(*vector, "Vector_SetElement")
  Vector__Lock(*vector\Mutex)
  With *vector
    If *vector\CurrentElement
      CallFunctionFast(\Destructor, \CurrentElement\Value)
      \CurrentElement\Value = CallFunctionFast(\Constructor, *Object)
    Else
      Vector__RaiseError("Null Pointer !", "Vector_SetElement")
    EndIf
  EndWith
  Vector__UnLock(*vector\Mutex)
EndProcedure
ProcedureDLL Vector_Set3(*vector.Vector, *Object, index.l)
  Vector__IsInitialized(*vector, "Vector_SetElement")
  Vector__Lock(*vector\Mutex)
  If Index <> #Vector_CurrentElement
    V__SelectElement(*vector, Index, #True, "Vector_SetElement")
  EndIf
  With *vector
    If *vector\CurrentElement
      CallFunctionFast(\Destructor, \CurrentElement\Value)
      \CurrentElement\Value = CallFunctionFast(\Constructor, *Object)
    Else
      Vector__RaiseError("Null Pointer !", "Vector_SetElement")
    EndIf
  EndWith
  Vector__UnLock(*vector\Mutex)
EndProcedure



ProcedureDLL Vector__SwapElements(*vector.Vector, Index1.l, Index2.l)
  Protected *node.VNode, Index.l, *node1.VNode, *node2.VNode, temp
  
  With *vector
    If Index1 <> Index2
      *node = \CurrentElement
      Index = \Index
      *node1 = V__SelectElement(*vector, Index1, #True, "Vector_SwapElements")
      *node2 = V__SelectElement(*vector, Index2, #True, "Vector_SwapElements")
      ;If *node1 And *node2
        temp = *node1\Value
        *node1\Value = *node2\Value
        *node2\Value = temp
      ;Else
      ;  Debug "Can't swap "+Str(Index1)+"<->"+Str(Index2)+" (Count: "+Str(*vector\Count)+")"
      ;EndIf
      \CurrentElement = *node
      \Index = Index
    EndIf
  EndWith
EndProcedure

ProcedureDLL Vector_SwapElements(*vector.Vector, Index1.l, Index2.l)
  Protected *node.VNode, Index.l, *node1.VNode, *node2.VNode, temp
  
  Vector__IsInitialized(*vector, "Vector_SwapElements")
  With *vector
    Vector__Lock(\Mutex)
    Vector__SwapElements(*vector, Index1, Index2)
    Vector__UnLock(\Mutex)
  EndWith
EndProcedure



ProcedureDLL Vector_SetConstructor(*vector.Vector, *function)
  Vector__IsInitialized(*vector, "Vector_SetConstructor")
  Vector__Lock(*vector\Mutex)
  *vector\Constructor = *function
  Vector__UnLock(*vector\Mutex)
EndProcedure

ProcedureDLL Vector_GetConstructor(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_GetConstructor")
  ProcedureReturn *vector\Constructor
EndProcedure

Procedure Vector__SetDestructor(*vector.Vector, *function)
  If *function
    *vector\Destructor = *function
  Else
    *vector\Destructor = @V__DefaultDestructor()
  EndIf
EndProcedure

ProcedureDLL Vector_SetDestructor(*vector.Vector, *function)
  Vector__IsInitialized(*vector, "Vector_SetDestructor")
  Vector__Lock(*vector\Mutex)
  Vector__SetDestructor(*vector, *function)
  Vector__UnLock(*vector\Mutex)
EndProcedure

ProcedureDLL Vector_GetDestructor(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_GetDestructor")
  ProcedureReturn *vector\Destructor
EndProcedure

ProcedureDLL Vector_SetComparator(*vector.Vector, *function)
  Vector__IsInitialized(*vector, "Vector_SetComparator")
  Vector__Lock(*vector\Mutex)
  *vector\Comparator = *function
  Vector__UnLock(*vector\Mutex)
EndProcedure

ProcedureDLL Vector_GetComparator(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_SetComparator")
  ProcedureReturn *vector\Comparator
EndProcedure





;/------------------------------------------\
;|                                          |
;-       Vector Constructor/Destructor
;|                                          |
;\------------------------------------------/



;{- Vector_Create
;   Create a Vector from a given Type
;   If the Type is known, Constructor and Destructor will be ignored
;   Otherwise, this 2 arguments are pointer to the Constructor and Destructor Functions
Procedure Vector__Create(Type.w, Size.l, *Constructor = #Null, *Destructor = #Null, *Comparator = #Null)
  Protected *vector.Vector

  If Type < #Vector_Type_Minimum Or Type > #Vector_Type_Maximum
    Vector__RaiseError("Incorrect Type value", "Vector_Create")
  EndIf
  *vector = Vector__Allocate(SizeOf(Vector))
  With *vector
    \Initilized = #Vector_Flag_Allocated
    \Type = Type
    \Destructor  = @V__DefaultDestructor()
    \Size = Size
    Select Type
      Case #Byte
        \Constructor = @V__DefaultConstructor()
        \Comparator  = @V__Comparator_Byte()
        \Size        = SizeOf(Byte)
      Case #Word
        \Constructor = @V__DefaultConstructor()
        \Comparator  = @V__Comparator_Word()
        \Size        = SizeOf(Word)
      Case #Long
        \Constructor = @V__DefaultConstructor()
        \Comparator  = @V__Comparator_Long()
        \Size        = SizeOf(Long)
      Case #Float
        \Constructor = @V__DefaultConstructor()
        \Comparator  = @V__Comparator_Float()
        \Size        = SizeOf(Float)
      Case #Quad
        \Constructor = @V__DefaultConstructor()
        \Comparator  = @V__Comparator_Quad()
        \Size        = SizeOf(Quad)
      Case #String
        \Constructor = @V__Constructor_String()
        \Comparator  = @V__Comparator_StringCaseSensitive() ; @V__Comparator_String()
        \Size        = 0
      Case #Double
        \Constructor = @V__DefaultConstructor()
        \Comparator  = @V__Comparator_Double()
        \Size        = SizeOf(Double)
      Default
        \Constructor = *Constructor
        Vector__SetDestructor(*vector, *Destructor)
        \Comparator  = *Comparator
    EndSelect
    \Count          = 0
    \Index          = -1
    \FirstElement   = #Null
    \LastElement    = #Null
    \CurrentElement = #Null
    \Mutex          = CreateMutex()
  EndWith
  AddElement(Vector__GarbageCollector()) : Vector__GarbageCollector() = *vector
  ProcedureReturn *vector
EndProcedure
ProcedureDLL Vector_CreateFromComplexStructure(*Constructor)
  ProcedureReturn Vector__Create(#VType_ComplexStructure, 0, *Constructor, #Null, #Null)
EndProcedure
ProcedureDLL Vector_CreateFromComplexStructure2(*Constructor, *Destructor)
  ProcedureReturn Vector__Create(#VType_ComplexStructure, 0, *Constructor, *Destructor, #Null)
EndProcedure
ProcedureDLL Vector_CreateFromComplexStructure3(*Constructor, *Destructor, *Comparator)
  ProcedureReturn Vector__Create(#VType_ComplexStructure, 0, *Constructor, *Destructor, *Comparator)
EndProcedure
ProcedureDLL Vector_CreateFromSimpleStructure(Size.l)
  If Size < 1
    Vector__RaiseError("Size must be positive !", "Vector_CreateFromSimpleStructure")
  EndIf
  ProcedureReturn Vector__Create(#VType_SimpleStructure, Size, @V__DefaultConstructor(), #Null, #Null)
EndProcedure
ProcedureDLL Vector_CreateFromSimpleStructure2(Size.l, *Comparator)
  If Size < 1
    Vector__RaiseError("Size must be positive !", "Vector_CreateFromSimpleStructure")
  EndIf
  ProcedureReturn Vector__Create(#VType_SimpleStructure, Size, @V__DefaultConstructor(), #Null, *Comparator)
EndProcedure
ProcedureDLL Vector_Create(Type)
  ProcedureReturn Vector__Create(Type, 0)
EndProcedure
;}

;{- Vector_Clear
;   Delete all nodes
ProcedureDLL Vector_Clear(*vector.Vector)
  Protected *cNode.VNode, *nNode.VNode
  
  Vector__IsInitialized(*vector, "Vector_Clear")
  With *vector
    Vector__Lock(\Mutex)
    *cNode = *vector\FirstElement
    While *cNode
      *nNode = *cNode\NextElement
      CallFunctionFast(\Destructor, *cNode\Value)
      *cNode = *nNode
    Wend
    \Count = 0
    \Index = -1
    \FirstElement = #Null
    \LastElement = #Null
    Vector__UnLock(\Mutex)
  EndWith
EndProcedure
;}

;{- Vector_Free
;   FreeMemory for the Vector and his Nodes
ProcedureDLL Vector_Free(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_Free")
  Vector_Clear(*vector)
  *vector\Initilized = #Null
  With *vector
    TryLockMutex(\Mutex)
    UnlockMutex(\Mutex)
    FreeMutex(\Mutex)
    \Mutex = #Null
  EndWith
  ForEach Vector__GarbageCollector()
    If Vector__GarbageCollector() = *vector
      DeleteElement(Vector__GarbageCollector())
      Break
    EndIf
  Next
  FreeMemory(*vector)
EndProcedure
;}

;{- Vector_FreeFromPointer
;   FreeMemory for the Vector and his Nodes, and reset the pointer to the Vector
ProcedureDLL Vector_FreeFromPointer(*ptr.Long)
  If *ptr And *ptr\l
    Vector_Free(*ptr\l)
    *ptr\l = #Null
  EndIf
EndProcedure
;}

;{- Vector_FreeGarbageCollector
;   Clear all Vector who haven't been cleared !!!
ProcedureDLL Vector_FreeGarbageCollector()
  While FirstElement(Vector__GarbageCollector())
    Vector_Free(Vector__GarbageCollector())
  Wend
  ClearList(Vector__GarbageCollector())
EndProcedure
;}







;/------------------------------------------\
;|                                          |
;-           Add/Delete Functions
;|                                          |
;\------------------------------------------/



;{- Vector_AddElement (after the Current/New Element)
Procedure Vector__AddElement(*vector.Vector, *Object = #Null, index.l = #Vector_CurrentElement)
  Protected *new.VNode

  If Index <> #Vector_CurrentElement
    V__SelectElement(*vector, Index, #True, "AddElement")
  EndIf
  
  With *vector
    *new                  = Vector__Allocate(SizeOf(VNode))
    If \Constructor = @V__DefaultConstructor()
      *new\Value          = CallFunctionFast(@V__DefaultConstructor(), \Size, *Object)
    Else
      *new\Value          = CallFunctionFast(\Constructor, *Object)
    EndIf
    *new\PreviousElement  = #Null
    *new\NextElement      = #Null
    
    If *vector\Count = 0      ; Add the firts Element
    
      \FirstElement   = *new
      \LastElement    = *new
      \Index          = 0
      
    ElseIf \Index = -1        ; Add at head of list
    
      *new\NextElement              = \FirstElement
      \FirstElement\PreviousElement = *new
      \FirstElement                 = *new
      \Index                        = 0
    
    ElseIf \Index >= \Count-1 ; Add at end of list
    
      *new\PreviousElement          = \LastElement
      \LastElement\NextElement      = *new
      \LastElement                  = *new
      \Index                        = \Count
    
    Else                      ; In the list
    
      *new\PreviousElement        = \CurrentElement
      *new\NextElement            = \CurrentElement\NextElement
      \CurrentElement\NextElement\PreviousElement =  *new
      \CurrentElement\NextElement =  *new
      \Index                      + 1
    
    EndIf
    \Count + 1
    \CurrentElement = *new
  EndWith
  ProcedureReturn *new

EndProcedure
ProcedureDLL Vector_AddElement(*vector.Vector)
  Protected retour
  Vector__IsInitialized(*vector, "Vector_AddElement")
  Vector__Lock(*vector\Mutex)
  retour = Vector__AddElement(*vector, #Null, #Vector_CurrentElement)
  Vector__UnLock(*vector\Mutex)
  ProcedureReturn retour
EndProcedure
ProcedureDLL Vector_AddElement2(*vector.Vector, *Object)
  Protected retour
  Vector__IsInitialized(*vector, "Vector_AddElement")
  Vector__Lock(*vector\Mutex)
  retour = Vector__AddElement(*vector, *Object, #Vector_CurrentElement)
  Vector__UnLock(*vector\Mutex)
  ProcedureReturn retour
EndProcedure
ProcedureDLL Vector_AddElement3(*vector.Vector, *Object, index.l)
  Protected retour
  Vector__IsInitialized(*vector, "Vector_AddElement")
  Vector__Lock(*vector\Mutex)
  retour = Vector__AddElement(*vector, *Object, Index)
  Vector__UnLock(*vector\Mutex)
  ProcedureReturn retour
EndProcedure
;}

;{- Vector_DeleteElement (the Current/Selected Element)
Procedure Vector__DeleteElement(*vector.Vector, index.l = #Vector_CurrentElement)
  Protected *cNode.VNode

  If Index <> #Vector_CurrentElement
    V__SelectElement(*vector, Index, #True, "DeleteElement")
  EndIf
  
  With *vector
    If \CurrentElement = #Null
      Vector__RaiseError("There is no current element", "Vector_DeleteElement")
    EndIf
    *cNode = \CurrentElement
    If *cNode\NextElement
      *cNode\NextElement\PreviousElement = *cNode\PreviousElement
    EndIf
    If *cNode\PreviousElement
      *cNode\PreviousElement\NextElement = *cNode\NextElement
    EndIf
    \CurrentElement = *cNode\PreviousElement
    \Index - 1
    \Count - 1
    If *cNode = \FirstElement
      \FirstElement = *cNode\NextElement
    EndIf
    If *cNode = \LastElement
      \LastElement = *cNode\PreviousElement
    EndIf
    If \Destructor
      CallFunctionFast(\Destructor, *cNode\Value)
    EndIf
    FreeMemory(*cNode\Value)
    FreeMemory(*cNode)
  EndWith
EndProcedure
ProcedureDLL Vector_DeleteElement(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_DeleteElement")
  Vector__Lock(*vector\Mutex)
  Vector__DeleteElement(*vector, #Vector_CurrentElement)
  Vector__UnLock(*vector\Mutex)
EndProcedure
ProcedureDLL Vector_DeleteElement2(*vector.Vector, index.l)
  Vector__IsInitialized(*vector, "Vector_DeleteElement")
  Vector__Lock(*vector\Mutex)
  Vector__DeleteElement(*vector, Index)
  Vector__UnLock(*vector\Mutex)
EndProcedure
;}



;/------------------------------------------\
;|                                          |
;-              Sort Functions
;|                                          |
;\------------------------------------------/


Procedure Vector__QS_Swap(*vector.Vector, iMin.l, iMax.l, opt.l)
  Protected *temp.Long, *max.VNode, *min.VNode, test1.b, test2.b
  
  If opt & #Vector_Sort_Downward
    test1 = -1: test2 =  1
  Else
    test1 =  1: test2 = -1
  EndIf
  
  *max = V__SelectElement(*vector, iMax)
  *temp = *max\Value
  V__SelectElement(*vector, iMin)
  
  With *vector
    While (iMax > iMin)
      While iMax > iMin And CallFunctionFast(\Comparator, \CurrentElement\Value, *temp) <> test1
        iMin + 1 : V__NextElement(*vector)
      Wend
      *min = \CurrentElement
      If iMax > iMin
        ; t[max] = t[min]
        *max\Value = *min\Value
        ; Index = max
        V__SetPosition(*vector, *max, iMax)
        ; max - 1
        iMax - 1 : V__PreviousElement(*vector)
        While iMax > iMin And CallFunctionFast(\Comparator, \CurrentElement\Value, *temp) <> test2
          iMax - 1 : V__PreviousElement(*vector)
        Wend
        *max = \CurrentElement
        If iMax > iMin
          ; t[min] = t[max]
          *min\Value = *max\Value
          ; Index = min
          V__SetPosition(*vector, *min, iMin)
          ; min + 1
          iMin + 1 : V__NextElement(*vector)
        EndIf
      EndIf
    Wend
  EndWith
  ; t[max] = temp;
  *max\Value = *temp
  ProcedureReturn iMax
EndProcedure



Procedure Vector__QS_Split(*vector.Vector, deb.l, fin.l, opt.l)
  Protected mil.l

  If deb < fin
    mil = Vector__QS_Swap(*vector, deb, fin, opt)
    If mil-deb > fin-mil
      Vector__QS_Split(*vector, mil+1, fin, opt)
      Vector__QS_Split(*vector, deb, mil-1, opt)
    Else
      Vector__QS_Split(*vector, deb, mil-1, opt)
      Vector__QS_Split(*vector, mil+1, fin, opt)
    EndIf
  EndIf
EndProcedure


ProcedureDLL Vector__SortOnRange(*vector.Vector, debut.l, fin.l, Options.l)
  With *vector
    Vector__Lock(\Mutex)
    \CurrentElement = \FirstElement
    \Index          = 0
    Vector__QS_Split(*vector, debut, fin, Options)
    \CurrentElement = \FirstElement
    \Index          = 0
    Vector__UnLock(\Mutex)
  EndWith
EndProcedure

ProcedureDLL Vector_SortOnRange(*vector.Vector, debut.l, fin.l)
  Vector__IsInitialized(*vector, "Vector_SortOnRange")
  Vector__SortOnRange(*vector, debut, fin, #Null)
EndProcedure
ProcedureDLL Vector_SortOnRange2(*vector.Vector, debut.l, fin.l, Options.l)
  Vector__IsInitialized(*vector, "Vector_SortOnRange")
  Vector__SortOnRange(*vector, debut, fin, Options)
EndProcedure

ProcedureDLL Vector_Sort(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_Sort")
  Vector__SortOnRange(*vector, 0, *vector\Count - 1, #Null)
EndProcedure
ProcedureDLL Vector_Sort2(*vector.Vector, Options.l)
  Vector__IsInitialized(*vector, "Vector_Sort")
  Vector__SortOnRange(*vector, 0, *vector\Count - 1, Options)
EndProcedure


ProcedureDLL Vector_Randomize(*vector.Vector)
  Protected i.l, nb.l

  Vector__IsInitialized(*vector, "Vector_Randomize")
  Vector__Lock(*vector\Mutex)
  nb = *vector\Count-1
  For i = nb To -1 Step -1
    Vector__SwapElements(*vector, Random(nb), Random(nb))
  Next i
  Vector__UnLock(*vector\Mutex)
EndProcedure



;/------------------------------------------\
;|                                          |
;-             Search Functions
;|                                          |
;\------------------------------------------/


; *index.Long is optional.
;   But if it's a valid pointer, it returns the index of the found object
Procedure Vector__Search(*vector.Vector, *element, *index.Long = #Null)
  Protected *backup, index.l = -1

  With *vector
    If \Comparator = #Null
      Vector__RaiseError("No comparator defined !", "Vector_Search")
    EndIf
    Vector__Reset(*vector)
    While index = -1 And Vector__NextElement(*vector)
      If CallFunctionFast(\Comparator, \CurrentElement\Value, *element) = 0
        index = \index
      EndIf
    Wend
    If *index
      *index\l = index
    EndIf
  EndWith
  If index > -1
    ProcedureReturn #True
  Else
    ProcedureReturn #False
  EndIf
EndProcedure
ProcedureDLL Vector_Search(*vector.Vector, *element)
  Protected retour
  
  Vector__IsInitialized(*vector, "Vector_Search")
  Vector__Lock(*vector\Mutex)
  retour = Vector__Search(*vector, *element, #Null)
  Vector__UnLock(*vector\Mutex)
  ProcedureReturn retour
EndProcedure
ProcedureDLL Vector_Search2(*vector.Vector, *element, *index.Long)
  Protected retour

  Vector__IsInitialized(*vector, "Vector_Search")
  Vector__Lock(*vector\Mutex)
  retour = Vector__Search(*vector, *element, *index)
  Vector__UnLock(*vector\Mutex)
  ProcedureReturn retour
EndProcedure





;/------------------------------------------\
;|                                          |
;-             Others Functions
;|                                          |
;\------------------------------------------/



ProcedureDLL Vector_Copy(*original.Vector)
  Protected *vector.Vector, *backup,    *s.String

  Vector__IsInitialized(*original, "Vector_Copy")
  With *original
    Vector__Lock(\Mutex)
    Vector__SaveContext(*original, *backup)
    *vector = Vector_Create(\Type)
    *vector\Constructor = \Constructor
    *vector\Destructor  = \Destructor
    Vector_Reset(*original)
    While Vector_NextElement(*original)
      If *vector\Type = #String
        *s = \CurrentElement\Value
        Vector__AddElement(*vector, @*s\s)
      Else
        Vector__AddElement(*vector, \CurrentElement\Value)
      EndIf
      *s = Vector_Get(*vector)
    Wend
    Vector__UnLock(\Mutex)
  EndWith
  Vector__RestoreContext(*original, *backup)
  ProcedureReturn *vector
EndProcedure

; Add Source Elements to Destination Vector
ProcedureDLL Vector_Combine(*destination.Vector, *source.Vector)
  Protected *backup.Vector

  Vector__IsInitialized(*source, "Vector_Combine")
  Vector__IsInitialized(*destination, "Vector_Combine")
  Vector__Lock(*source\Mutex)
  Vector__Lock(*destination\Mutex)
  If Vector_SameType(*destination,*source)
    Vector__SaveContext(*source, *backup)
    Vector__Reset(*source)
    While Vector__NextElement(*source)
      Vector__AddElement(*destination, *source\CurrentElement\Value)
    Wend
    Vector__RestoreContext(*source, *backup)
  Else
    Vector__RaiseError("Incompatibles Vectors !", "Vector_Combine")
  EndIf
  Vector__UnLock(*destination\Mutex)
  Vector__UnLock(*source\Mutex)
EndProcedure


ProcedureDLL Vector_Exclude(*source.Vector, *mask.Vector)
  Protected *backup.Vector

  Vector__IsInitialized(*source, "Vector_Exclude")
  Vector__IsInitialized(*mask, "Vector_Exclude")
  Vector__Lock(*source\Mutex)
  Vector__Lock(*mask\Mutex)
  If Vector_SameType(*mask,*source)
    Vector__Reset(*source)
    While Vector__NextElement(*source)
      If Vector__Search(*mask, *source\CurrentElement\Value)
        Vector__DeleteElement(*source)
      EndIf
    Wend
    Vector_Reset(*source)
  Else
    Vector__RaiseError("Incompatibles Vectors !", "Vector_Exclude")
  EndIf
  Vector__UnLock(*mask\Mutex)
  Vector__UnLock(*source\Mutex)
EndProcedure




;/------------------------------------------\
;|                                          |
;-            Directory Functions
;|                                          |
;\------------------------------------------/

Procedure Vector__NextDirectoryEntry(hDir.l)
  Protected Type.l, Name$
  
  If NextDirectoryEntry(hDir)
    Type = DirectoryEntryType(hDir)
    If Type = #PB_DirectoryEntry_Directory
      Name$ = DirectoryEntryName(hDir)
      If Name$ = "." Or Name$ = ".."
        ProcedureReturn Vector__NextDirectoryEntry(hDir)
      EndIf
    EndIf
    ProcedureReturn Type
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure

Macro Vector__ScanDir__Beginning()
  Protected hDir.l, FileName$, Type.l, RelativeName$, CompleteName$, FileExt$, FileInfo.Vector_FileInfos
  
  hDir = ExamineDirectory(#PB_Any, Dir$+SecondDirPart$, "*.*")
  If hDir
    Type = Vector__NextDirectoryEntry(hDir)
    While Type ; And *Vector\Stop = #Null
      FileName$ = DirectoryEntryName(hDir)
      FileExt$  = GetExtensionPart(FileName$)
      If Len(FileExt$) = 0
        FileExt$ = "*"
      EndIf
      ;Debug FileName$
      RelativeName$ = SecondDirPart$ + FileName$
      If KeepAllPath
        CompleteName$ = Dir$+RelativeName$
      Else
        CompleteName$ = RelativeName$
      EndIf
EndMacro
Macro Vector__ScanDir__End()
      Type = Vector__NextDirectoryEntry(hDir)
    Wend
    FinishDirectory(hDir)
  Else
    Debug "Can't examine directory : " + Dir$
  EndIf
EndMacro


Procedure Vector__ScanDir(*vector.Vector, Dir$, SecondDirPart$, Options.l, ext$, KeepAllPath)
    Vector__ScanDir__Beginning()
      If Type = #PB_DirectoryEntry_File
        If ext$ = ";;" Or FindString(ext$, ";"+LCase(FileExt$)+";", 1)
          Vector__AddElement(*vector, @CompleteName$)
        EndIf
      Else
        If Options & #Vector_Dir_KeepFolder
          Vector__AddElement(*vector, @CompleteName$)
        EndIf
        If Options & #Vector_Dir_Recursive
          Vector__ScanDir(*vector, Dir$, RelativeName$+#PATH_DELIMITER, Options, ext$, KeepAllPath)
        EndIf
      EndIf
    Vector__ScanDir__End()
EndProcedure

; extensions$ : list of used extensions dlimited by ";"
; Return Null Pointer, if path not exist
Procedure Vector__ScanDirectory(Directory$, Options.l = #Null, extensions$ = #NULL$, KeepAllPath = #False)
  Protected *vector.Vector
  If FileSize(Directory$) = -2
    If Right(Directory$, 1) <> #PATH_DELIMITER
      Directory$ + #PATH_DELIMITER
    EndIf
    If KeepAllPath
      If Left(Directory$, 1) <> "/" And Mid(Directory$, 2, 1) <> ":" And Left(Directory$, 2) <> "\\"
        Directory$ = GetCurrentDirectory() + Directory$
      EndIf
    EndIf
    ;Debug Directory$
    *vector = Vector_Create(#String)
    Vector__ScanDir(*vector, Directory$, "", Options, ";"+LCase(extensions$)+";", KeepAllPath)
  Else
    Vector__RaiseError("Folder doesn't exists: " + Directory$, "Vector_ScanDirectory")
  EndIf
  ProcedureReturn *vector
EndProcedure

ProcedureDLL Vector_ScanDirectory(Directory$)
  ProcedureReturn Vector__ScanDirectory(Directory$)
EndProcedure
ProcedureDLL Vector_ScanDirectory2(Directory$, Options.l)
  ProcedureReturn Vector__ScanDirectory(Directory$, Options)
EndProcedure
ProcedureDLL Vector_ScanDirectory3(Directory$, Options.l, extensions$)
  ProcedureReturn Vector__ScanDirectory(Directory$, Options, extensions$)
EndProcedure
ProcedureDLL Vector_ScanDirectory4(Directory$, Options.l, extensions$, KeepAllPath.l)
  ProcedureReturn Vector__ScanDirectory(Directory$, Options, extensions$, KeepAllPath)
EndProcedure




;/------------------------------------------\
;|                                          |
;-       Advanced Directory Functions
;|                                          |
;\------------------------------------------/


ProcedureDLL Vector_Sort_FileInfos(*vector.Vector, SortMode.l = #Vector_Sort_Name_Upward)
  Select SortMode
    Case #Vector_Sort_Name_Upward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_Name_U())
    Case #Vector_Sort_Name_Downward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_Name_D())
    Case #Vector_Sort_FileSize_Upward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_Size_U())
    Case #Vector_Sort_FileSize_Downward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_Size_D())
    Case #Vector_Sort_DateCreated_Upward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_DateC_U())
    Case #Vector_Sort_DateCreated_Downward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_DateC_D())
    Case #Vector_Sort_DateAccessed_Upward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_DateA_U())
    Case #Vector_Sort_DateAccessed_Downward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_DateA_D())
    Case #Vector_Sort_DateModified_Upward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_DateM_U())
    Case #Vector_Sort_DateModified_Downward
      Vector_SetComparator(*vector, @V__Comparator_FileInfo_DateM_D())
    Default
      Vector__RaiseError("Incorrect 'Option' value", "Vector_Sort_FileInfos")
  EndSelect
  Vector_Sort(*vector)
EndProcedure

Procedure Vector__ScanDirAdv(*vector.Vector, Dir$, SecondDirPart$, Options.l, ext$)
  Protected KeepAllPath.b = #True
  Vector__ScanDir__Beginning()
  FileInfo\ShortFileName    = RelativeName$
  FileInfo\CompleteFileName = CompleteName$
  FileInfo\FileSize         = DirectoryEntrySize(hDir)
  FileInfo\DateCreated      = DirectoryEntryDate(hDir, #PB_Date_Created)
  FileInfo\DateAccessed     = DirectoryEntryDate(hDir, #PB_Date_Accessed)
  FileInfo\DateModified     = DirectoryEntryDate(hDir, #PB_Date_Modified)
  If Type = #PB_DirectoryEntry_File
    If ext$ = ";;" Or FindString(ext$, ";"+LCase(FileExt$)+";", 1)
      Vector__AddElement(*vector, FileInfo)
    EndIf
  Else
    If Options & #Vector_Dir_KeepFolder
      FileInfo\FileSize = -2
      Vector__AddElement(*vector, FileInfo)
    EndIf
    If Options & #Vector_Dir_Recursive
      Vector__ScanDirAdv(*vector, Dir$, RelativeName$+#PATH_DELIMITER, Options, ext$)
    EndIf
  EndIf
  Vector__ScanDir__End()
EndProcedure

; extensions$ : list of used extensions dlimited by ";"
; Return Null Pointer, if path not exist
Procedure Vector__ScanDirectoryAdv(Directory$, Options.l = #Null, extensions$ = #NULL$, Sort.l = #Vector_Sort_Name_Upward)
  Protected *vector.Vector_FileInfos
  If FileSize(Directory$) = -2
    If Right(Directory$, 1) <> #PATH_DELIMITER
      Directory$ + #PATH_DELIMITER
    EndIf
    ;If KeepAllPath
      If Left(Directory$, 1) <> "/" And Mid(Directory$, 2, 1) <> ":"
        Directory$ = GetCurrentDirectory() + Directory$
      EndIf
    ;EndIf
    *vector = Vector_CreateFromComplexStructure2(@V__Constructor_FileInfos(), @V_Destructor_FileInfos())
    Vector__ScanDirAdv(*vector, Directory$, "", Options, ";"+LCase(extensions$)+";")
    If Sort <> #Vector_Sort_Name_Upward
      Vector_Sort_FileInfos(*vector, Sort)
    EndIf
  Else
    Vector__RaiseError("Folder doesn't exists: " + Directory$, "Vector_ScanDirectory")
  EndIf
  ProcedureReturn *vector
EndProcedure

ProcedureDLL Vector_ScanDirectoryAdvanced(Directory$)
  ProcedureReturn Vector__ScanDirectoryAdv(Directory$)
EndProcedure
ProcedureDLL Vector_ScanDirectoryAdvanced2(Directory$, Options.l)
  ProcedureReturn Vector__ScanDirectoryAdv(Directory$, Options)
EndProcedure
ProcedureDLL Vector_ScanDirectoryAdvanced3(Directory$, Options.l, extensions$)
  ProcedureReturn Vector__ScanDirectoryAdv(Directory$, Options, extensions$)
EndProcedure
ProcedureDLL Vector_ScanDirectoryAdvanced4(Directory$, Options.l, extensions$, SortMode.l)
  ProcedureReturn Vector__ScanDirectoryAdv(Directory$, Options, extensions$, SortMode.l)
EndProcedure






;/------------------------------------------\
;|                                          |
;-             String Functions
;|                                          |
;\------------------------------------------/


ProcedureDLL Vector_Explode(String$, Separator$)
  Protected *vector.Vector, i.l, chaine$
  
  *vector = Vector_Create(#String)
  For i = 1 To CountString(String$, Separator$)+1
    chaine$ = StringField(String$, i, Separator$)
    Vector__AddElement(*vector, @chaine$)
  Next i
  
  ProcedureReturn *vector
EndProcedure

ProcedureDLL.s Vector_Implode(*vector.Vector, Separator$)
  Protected String$, *s.String
  
  Vector__IsType(*vector, #String, "Vector_Implode")
  Vector_Reset(*vector)
  While Vector_NextElement(*vector)
    *s = Vector_Get(*vector)
    String$ + *s\s
    If *vector\CurrentElement\NextElement
      String$ + Separator$
    EndIf
  Wend

  ProcedureReturn String$
EndProcedure




;/------------------------------------------\
;|                                          |
;-             Files Functions
;|                                          |
;\------------------------------------------/


ProcedureDLL Vector_LoadByteFile(FileName$)
  Protected *vector.Vector, hFile.l, Value.b
  
  hFile = ReadFile(#PB_Any, FileName$)
  If hFile
    *vector = Vector_Create(#Byte)
    While Not(Eof(hFile))
      Value = ReadByte(hFile)
      Vector__AddElement(*vector, @Value)
    Wend
    CloseFile(hFile)
    ProcedureReturn *vector
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
ProcedureDLL Vector_LoadWordFile(FileName$)
  Protected *vector.Vector, hFile.l, Value.w
  
  hFile = ReadFile(#PB_Any, FileName$)
  If hFile
    *vector = Vector_Create(#Word)
    While Not(Eof(hFile))
      Value = ReadByte(hFile)
      Vector__AddElement(*vector, @Value)
    Wend
    CloseFile(hFile)
    ProcedureReturn *vector
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
ProcedureDLL Vector_LoadLongFile(FileName$)
  Protected *vector.Vector, hFile.l, Value.l
  
  hFile = ReadFile(#PB_Any, FileName$)
  If hFile
    *vector = Vector_Create(#Byte)
    While Not(Eof(hFile))
      Value = ReadLong(hFile)
      Vector__AddElement(*vector, @Value)
    Wend
    CloseFile(hFile)
    ProcedureReturn *vector
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
Declare Vector_LoadStringFile2(FileName$, KeepEmptyLine)
ProcedureDLL Vector_LoadStringFile(FileName$)
  ProcedureReturn Vector_LoadStringFile2(FileName$, #True)
EndProcedure
ProcedureDLL Vector_LoadStringFile2(FileName$, KeepEmptyLine)
  Protected *vector.Vector, hFile.l, line$
  
  hFile = ReadFile(#PB_Any, FileName$)
  If hFile
    *vector = Vector_Create(#String)
    While Not(Eof(hFile))
      line$ = ReadString(hFile)
      If line$ Or KeepEmptyLine
        Vector__AddElement(*vector, @line$)
      EndIf
    Wend
    CloseFile(hFile)
    ProcedureReturn *vector
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
ProcedureDLL Vector_LoadQuadFile(FileName$)
  Protected *vector.Vector, hFile.l, Value.q
  
  hFile = ReadFile(#PB_Any, FileName$)
  If hFile
    *vector = Vector_Create(#Byte)
    While Not(Eof(hFile))
      Value = ReadQuad(hFile)
      Vector__AddElement(*vector, @Value)
    Wend
    CloseFile(hFile)
    ProcedureReturn *vector
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
ProcedureDLL Vector_LoadFloatFile(FileName$)
  Protected *vector.Vector, hFile.l, Value.f
  
  hFile = ReadFile(#PB_Any, FileName$)
  If hFile
    *vector = Vector_Create(#Byte)
    While Not(Eof(hFile))
      Value = ReadFloat(hFile)
      Vector__AddElement(*vector, @Value)
    Wend
    CloseFile(hFile)
    ProcedureReturn *vector
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
ProcedureDLL Vector_LoadDoubleFile(FileName$)
  Protected *vector.Vector, hFile.l, Value.d
  
  hFile = ReadFile(#PB_Any, FileName$)
  If hFile
    *vector = Vector_Create(#Byte)
    While Not(Eof(hFile))
      Value = ReadDouble(hFile)
      Vector__AddElement(*vector, @Value)
    Wend
    CloseFile(hFile)
    ProcedureReturn *vector
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure


Procedure Vector__SaveFile(FileName$, *vector.Vector, FileFormat.l)
  Protected hFile.l, *str.String, *backup.Vector
  
  Vector__IsInitialized(*vector, "Vector_SaveFile")
  If *vector\Type = #VType_ComplexStructure
    Vector__RaiseError("You can't use this function to save a Complex Vector !", "Vector_SaveFile")
  EndIf
  hFile = CreateFile(#PB_Any, FileName$)
  If hFile
    Vector__SaveContext(*vector, *backup)
    Vector__Reset(*vector)
    While Vector__NextElement(*vector)
      Select *vector\Type
        Case #VType_SimpleStructure, #Byte, #Word, #Long, #Float, #Quad, #Double
          WriteData(hFile, *vector\CurrentElement\Value, *vector\Size)
        Case #String
          *str = V__Get(*vector)
          WriteStringN(hFile, *str\s, FileFormat)
      EndSelect
    Wend
    Vector__RestoreContext(*vector, *backup)
    CloseFile(hFile)
    ProcedureReturn #True
  Else
    ProcedureReturn #Null
  EndIf
EndProcedure
ProcedureDLL Vector_SaveFile(FileName$, *vector.Vector)
  Vector__SaveFile(FileName$, *vector, #PB_UTF8)
EndProcedure
ProcedureDLL Vector_SaveFile2(FileName$, *vector.Vector, FileFormat.l)
  Vector__SaveFile(FileName$, *vector, FileFormat.l)
EndProcedure




;/------------------------------------------\
;|                                          |
;-              Debug Functions
;|                                          |
;\------------------------------------------/



Macro V_Debug(val_)
  number$ + val_ + #Vector_NewLine
EndMacro

Declare.s Vector_Parse3(*vector.Vector, spacer$ = #Null$, number.b = #Null)
ProcedureDLL.s Vector_Parse(*vector.Vector)
  ProcedureReturn Vector_Parse3(*vector.Vector, #Null$, #Null)
EndProcedure
ProcedureDLL.s Vector_Parse2(*vector.Vector, spacer$)
  ProcedureReturn Vector_Parse3(*vector.Vector, spacer$, #Null)
EndProcedure
ProcedureDLL.s Vector_Parse3(*vector.Vector, spacer$ = #Null$, number.b = #Null)
  Protected nb, number$, *b.Byte, *w.Word, *l.Long, *f.Float, *q.Quad, *s.String, *d.Double
  Protected retour$
  Debug "Parse...."
  Vector__IsInitialized(*vector, "Vector_Debug")
  With *vector
    Vector__Lock(\Mutex)
    If \Type = #VType_SimpleStructure Or \Type = #VType_ComplexStructure
      Vector__RaiseError("Can't Debug Complexe Type !", "Vector_Debug")
    EndIf
    
    If number
      If \Count
        nb = Len(Str(*vector\Count-1))
      Else
        nb = 0
      EndIf
    EndIf
    
    Vector__Reset(*vector)
    While Vector__NextElement(*vector)
      If number
        number$ = spacer$ + RSet(Str(\Index), nb, "0") + ") "
      Else
        number$ = spacer$
      EndIf
      Select *vector\Type
        Case #Byte
          *b = V__Get(*Vector)
          retour$ + V_Debug(Str(*b\b))
        Case #Word
          *w = V__Get(*Vector)
          retour$ + V_Debug(Str(*w\w))
        Case #Long
          *l = V__Get(*Vector)
          retour$ + V_Debug(Str(*l\l))
        Case #Float
          *f = V__Get(*Vector)
          retour$ + V_Debug(StrF(*f\f))
        Case #Quad
          *q = V__Get(*Vector)
          retour$ + V_Debug(StrQ(*q\q))
        Case #String
          *s = V__Get(*Vector)
          retour$ + V_Debug(*s\s)
        Case #Double
          *d = V__Get(*Vector)
          retour$ + V_Debug(StrD(*d\d))
      EndSelect
    Wend
    Vector__UnLock(\Mutex)
  EndWith
  ProcedureReturn retour$
EndProcedure

Procedure Vector_Debug(*vector.Vector, spacer$ = #Null$, number.b = #Null)
  Protected String$, i.l
  String$ = Vector_Parse3(*vector, spacer$, number)
  For i = 1 To CountString(String$, #Vector_NewLine)
    Debug StringField(String$, i, #Vector_NewLine)
  Next i
EndProcedure



;/------------------------------------------\
;|                                          |
;-                  Macros
;|                                          |
;\------------------------------------------/


ProcedureDLL Vector_GetIndex(*vector.Vector)
  ProcedureReturn Vector_Index(*vector)
EndProcedure

ProcedureDLL Vector_First(*vector.Vector)
  ProcedureReturn Vector_FirstElement(*vector)
EndProcedure
ProcedureDLL Vector_Last(*vector.Vector)
  ProcedureReturn Vector_LastElement(*vector)
EndProcedure
ProcedureDLL Vector_Next(*vector.Vector)
  ProcedureReturn Vector_NextElement(*vector)
EndProcedure
ProcedureDLL Vector_Previous(*vector.Vector)
  ProcedureReturn Vector_PreviousElement(*vector)
EndProcedure
ProcedureDLL Vector_Swap(*vector.Vector, Idx1, Idx2)
  ProcedureReturn Vector_SwapElements(*vector, Idx1, Idx2)
EndProcedure


ProcedureDLL Vector_Add(*vector.Vector)
  ProcedureReturn Vector_AddElement(*vector)
EndProcedure
ProcedureDLL Vector_Add2(*vector.Vector, object)
  ProcedureReturn Vector_AddElement2(*vector, object)
EndProcedure
ProcedureDLL Vector_Add3(*vector.Vector, object, index)
  ProcedureReturn Vector_AddElement3(*vector, object, Index)
EndProcedure
ProcedureDLL Vector_AddAtStart(*vector.Vector)
  ProcedureReturn Vector_AddElement3(*vector, #Null, -1)
EndProcedure
ProcedureDLL Vector_AddAtStart2(*vector.Vector, object)
  ProcedureReturn Vector_AddElement3(*vector, object, -1)
EndProcedure
ProcedureDLL Vector_AddAtEnd(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_AddAtEnd")
  ProcedureReturn Vector_AddElement3(*vector, #Null, *vector\Count - 1)
EndProcedure
ProcedureDLL Vector_AddAtEnd2(*vector.Vector, *Object)
  Vector__IsInitialized(*vector, "Vector_AddAtEnd")
  ProcedureReturn Vector_AddElement3(*vector, *Object, *vector\Count - 1)
EndProcedure


ProcedureDLL Vector_Delete(*vector.Vector)
  Vector_DeleteElement(*vector)
EndProcedure
ProcedureDLL Vector_Delete2(*vector.Vector, index)
  Vector_DeleteElement2(*vector, Index)
EndProcedure
ProcedureDLL Vector_DeleteLastElement(*vector.Vector)
  Vector__IsInitialized(*vector, "Vector_DeleteLastElement")
  Vector_DeleteElement2(*vector, *vector\Count - 1)
EndProcedure
ProcedureDLL Vector_DeleteFirstElement(*vector.Vector)
  Vector_DeleteElement2(*vector, -1)
EndProcedure



ProcedureDLL Vector_Del(*vector.Vector)
  Vector_DeleteElement(*vector)
EndProcedure
ProcedureDLL Vector_Del2(*vector.Vector, index.l)
  Vector_DeleteElement2(*vector, Index)
EndProcedure
ProcedureDLL Vector_Select(*vector.Vector, index.l)
  ProcedureReturn Vector_SelectElement(*vector, Index)
EndProcedure
ProcedureDLL Vector_Select2(*vector.Vector, index.l, Fail.l)
  ProcedureReturn Vector_SelectElement2(*vector, Index, #Null)
EndProcedure
ProcedureDLL Vector_GetElement(*vector.Vector)
  ProcedureReturn Vector_Get(*vector)
EndProcedure
ProcedureDLL Vector_GetElement2(*vector.Vector, index.l)
  ProcedureReturn Vector_Get2(*vector, Index)
EndProcedure
ProcedureDLL Vector_GetElement3(*vector.Vector, index.l, Fail.l)
  ProcedureReturn Vector_Get3(*vector, Index, Fail)
EndProcedure
ProcedureDLL Vector_SetElement(*vector.Vector)
  Vector_Set(*vector)
EndProcedure
ProcedureDLL Vector_SetElement2(*vector.Vector, object)
  Vector_Set2(*vector, object)
EndProcedure
ProcedureDLL Vector_SetElement3(*vector.Vector, object, index.l)
  Vector_Set3(*vector, object, Index)
EndProcedure


ProcedureDLL Vector_Clone(*vector.Vector)
  ProcedureReturn Vector_Copy(*vector)
EndProcedure


ProcedureDLL Vector_SetConstructorFunction(*vector.Vector, function)
  Vector_SetConstructor(*vector, function)
EndProcedure
ProcedureDLL Vector_SetDestructorFunction(*vector.Vector, function)
  Vector_SetDestructor(*vector, function)
EndProcedure
ProcedureDLL Vector_SetComparatorFunction(*vector.Vector, function)
  Vector_SetComparator(*vector, function)
EndProcedure
ProcedureDLL Vector_GetConstructorFunction(*vector.Vector)
  ProcedureReturn Vector_GetConstructor(*vector)
EndProcedure
ProcedureDLL Vector_GetDestructorFunction(*vector.Vector)
  ProcedureReturn Vector_GetDestructor(*vector)
EndProcedure
ProcedureDLL Vector_GetComparatorFunction(*vector.Vector)
  ProcedureReturn Vector_GetComparator(*vector)
EndProcedure





;/------------------------------------------\
;|                                          |
;-        Check for Internal Errors
;|                                          |
;\------------------------------------------/


Procedure Vector__TestIntegrite(*vector.Vector)
  Protected NewList ll.VNode(), Index.l, *node.VNode
  
  With *vector\CurrentElement
    Vector__Reset(*vector)
    While Vector__NextElement(*vector)
      AddElement(ll())
      ;Debug "------"+Str(*vector\Index)+"-------"
      ;Debug "prev : " + Str(\PreviousElement)
      ;Debug "curr : " + Str(*vector\CurrentElement)
      ;Debug "next : " + Str(\NextElement)
      ll()\NextElement = \NextElement
      ll()\PreviousElement = \PreviousElement
      ll()\Value = *vector\CurrentElement
    Wend
  EndWith
  
  ResetList(ll())
  While NextElement(ll())
    Index = ListIndex(ll())
    *node = ll()
    If Index > 0
      PreviousElement(ll())
      If *node\PreviousElement <> ll()\Value
        ;Debug ll()\Value
        Vector__RaiseError("1)Internal vector errors", "V__TestIntegrite")
      EndIf
      NextElement(ll())
    EndIf
    
    If Index < CountList(ll())-1
      NextElement(ll())
      If *node\NextElement <> ll()\Value
        ;Debug ll()\Value
        Vector__RaiseError("1)Internal vector errors", "V__TestIntegrite")
      EndIf
      PreviousElement(ll())
    EndIf
  Wend
EndProcedure

Procedure Vector__TestPosition(*vector.Vector, Index.l, *node.VNode, indice.l)
  Vector__Reset(*vector)
  While Vector__NextElement(*vector)
    If *vector\Index = Index
      If *vector\CurrentElement = *node
        ProcedureReturn
      EndIf
      Vector__RaiseError("Position Problem !")
    EndIf
  Wend
  Vector__RaiseError("Don't find !")
EndProcedure



DisableExplicit



; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 1680
; FirstLine = 1663
; Folding = ---------------------------------